<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/27/2009 *
 * ===================                          Last change: 10/27/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : filter-functions.php                             *
 * -------------------------------------------------------------------- *
 * Short description : Functions for filter system                      *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer Filter-System                    *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Init "generic filter system"
function initFilterSystem () {
	// Is the filter already initialized?
	if (isset($GLOBALS['filter_init'])) {
		// Then abort here
		reportBug(__FUNCTION__, __LINE__, 'Filter system already initialized.');
	} // END - if

	// Load all saved filers if ext-sql_patches is updated
	if ((isset($GLOBALS['cache_array']['filter']['filter_name'])) && (!isset($GLOBALS['cache_array']['filter']['chains']))) {
		// Prepare filter array
		prepareFilterArray();

		// Mark it as initialized
		$GLOBALS['filter_init'] = TRUE;
	} elseif ((!isInstallationPhase()) && (isExtensionInstalledAndNewer('sql_patches', '0.5.9'))) {
		// Init add
		$add = '';
		if (isExtensionINstalledAndNewer('sql_patches', '0.6.0')) {
			$add = ',`filter_counter`';
		} // END - if

		// Load all filters
		$result = SQL_QUERY('SELECT
	`filter_name`,
	`filter_function`,
	`filter_active`
	' . $add . '
FROM
	`{?_MYSQL_PREFIX?}_filters`
ORDER BY
	`filter_id` ASC', __FUNCTION__, __LINE__);

		// Are there entries?
		if (!SQL_HASZERONUMS($result)) {
			// Load all filters
			while ($filterArray = SQL_FETCHARRAY($result)) {
				// Get filter name and function
				$filterName     = $filterArray['filter_name'];
				$filterFunction = $filterArray['filter_function'];

				// Set counter to default
				$GLOBALS['cache_array']['filter']['counter'][$filterName][$filterFunction] = '0';

				// Mark this filter as loaded (from database)
				$GLOBALS['cache_array']['filter']['loaded'][$filterName][$filterFunction] = TRUE;

				// Set this filter
				$GLOBALS['cache_array']['filter']['chains'][$filterName][$filterFunction] = $filterArray['filter_active'];

				// Is the array element for counter there?
				if (isset($filterArray['filter_counter'])) {
					// Then use this value!
					$GLOBALS['cache_array']['filter']['counter'][$filterName][$filterFunction] = $filterArray['filter_counter'];
				} // END - if
			} // END - while
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	}

	// Init filters
	registerFilter(__FUNCTION__, __LINE__, 'init', 'LOAD_CONFIGURATION');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'INIT_RANDOMIZER');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'LOAD_RUNTIME_INCLUDES');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'INIT_EXTENSIONS');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'INIT_SESSION');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'SET_CURRENT_DATE');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'INIT_RANDOM_NUMBER');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'CHECK_REPOSITORY_REVISION');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'RUN_HOURLY_RESET');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'RUN_DAILY_RESET');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'TRIGGER_SENDING_POOL');
	// @TODO Remove this forced removal after a year or so
	unregisterFilter(__FUNCTION__, __LINE__, 'init', 'DETERMINE_USERNAME', TRUE);
	registerFilter(__FUNCTION__, __LINE__, 'init', 'DETERMINE_WHAT_ACTION');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'COUNT_MODULE');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'UPDATE_LOGIN_DATA');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'ACTIVATE_EXCHANGE');
	registerFilter(__FUNCTION__, __LINE__, 'init', 'REDIRECT_WRONG_SERVER_NAME');

	// Post-initialization
	registerFilter(__FUNCTION__, __LINE__, 'post_init', 'DETERMINE_USERNAME');

	// Page headers - pre-filter (normally, you want to register here)
	registerFilter(__FUNCTION__, __LINE__, 'pre_page_header', 'LOAD_PAGE_HEADER');

	// Page headers - post-filter (normally, you don't want to register here)
	//-------------------- LAST FILTER FOR THIS CHAIN! ------------------------
	registerFilter(__FUNCTION__, __LINE__, 'post_page_header', 'FINISH_PAGE_HEADER');
	//-------------------- LAST FILTER FOR THIS CHAIN! ------------------------

	// 'You are here' navigation - post filter
	registerFilter(__FUNCTION__, __LINE__, 'post_youhere_line', 'CALL_HANDLER_LOGIN_FAILTURES');
	registerFilter(__FUNCTION__, __LINE__, 'post_youhere_line', 'HANDLE_HOME_IN_INDEX_SETTING');

	// Filters for pre-extension-registration
	registerFilter(__FUNCTION__, __LINE__, 'pre_extension_installed', 'RUN_SQLS');

	// Filters for post-extension-registration
	registerFilter(__FUNCTION__, __LINE__, 'post_extension_installed', 'AUTO_ACTIVATE_EXTENSION');
	registerFilter(__FUNCTION__, __LINE__, 'post_extension_installed', 'SOLVE_TASK');
	registerFilter(__FUNCTION__, __LINE__, 'post_extension_installed', 'LOAD_INCLUDES');
	registerFilter(__FUNCTION__, __LINE__, 'post_extension_installed', 'REMOVE_UPDATES');
	registerFilter(__FUNCTION__, __LINE__, 'post_extension_installed', 'EXTENSION_MARK_INSTALLED');

	// Solving tasks
	registerFilter(__FUNCTION__, __LINE__, 'solve_task', 'SOLVE_TASK');

	// Loading includes in general
	registerFilter(__FUNCTION__, __LINE__, 'load_includes', 'LOAD_INCLUDES');

	// Run SQLs
	registerFilter(__FUNCTION__, __LINE__, 'run_sqls', 'RUN_SQLS');

	// Admin ACL check
	registerFilter(__FUNCTION__, __LINE__, 'check_admin_acl', 'CHECK_ADMIN_ACL');

	// Register shutdown filters
	registerFilter(__FUNCTION__, __LINE__, 'shutdown', 'FLUSH_FILTERS');
	registerFilter(__FUNCTION__, __LINE__, 'shutdown', 'FLUSH_STATS');
	registerFilter(__FUNCTION__, __LINE__, 'shutdown', 'FLUSH_TEMPLATE_CACHE');
	registerFilter(__FUNCTION__, __LINE__, 'shutdown', 'FLUSH_OUTPUT');

	// Compiling code
	registerFilter(__FUNCTION__, __LINE__, 'compile_code', 'COMPILE_CONFIG');
	registerFilter(__FUNCTION__, __LINE__, 'compile_code', 'COMPILE_EXPRESSION_CODE');

	// Generic extension update filters
	registerFilter(__FUNCTION__, __LINE__, 'extension_update', 'UPDATE_EXTENSION_DATA');

	// Do hourly reset stuff, keep this entry first in this chain:
	registerFilter(__FUNCTION__, __LINE__, 'hourly', 'RUN_HOURLY_INCLUDES');

	// Do reset stuff, keep this entry first in this chain:
	registerFilter(__FUNCTION__, __LINE__, 'reset', 'RUN_RESET_INCLUDES');

	// Remove extension
	registerFilter(__FUNCTION__, __LINE__, 'extension_remove', 'REMOVE_EXTENSION');

	// Exclude some users
	registerFilter(__FUNCTION__, __LINE__, 'exclude_users', 'HTML_INCLUDE_USERS');

	// Handling of fatal errors
	registerFilter(__FUNCTION__, __LINE__, 'handle_fatal_errors', 'HANDLE_FATAL_ERRORS');

	// Page footer filters
	registerFilter(__FUNCTION__, __LINE__, 'page_footer', 'HANDLE_FATAL_ERRORS');
	registerFilter(__FUNCTION__, __LINE__, 'page_footer', 'DISPLAY_COPYRIGHT');
	registerFilter(__FUNCTION__, __LINE__, 'page_footer', 'DISPLAY_PARSING_TIME');

	// Member login check. Always keep FETCH_USER_DATA as first entry!
	registerFilter(__FUNCTION__, __LINE__, 'member_login_check', 'FETCH_USER_DATA');

	// Admin login
	registerFilter(__FUNCTION__, __LINE__, 'do_admin_login_done', 'DO_LOGIN_ADMIN');

	// Admin mail links
	registerFilter(__FUNCTION__, __LINE__, 'generate_admin_mail_links', 'GENERATE_POOL_MAIL_LINKS');

	// Build mails
	registerFilter(__FUNCTION__, __LINE__, 'send_build_mail', 'SEND_BUILD_MAIL');

	// Handle referral banner click/view
	registerFilter(__FUNCTION__, __LINE__, 'handle_click_php', 'HANDLE_REFERRER_BANNER_CLICK');
	registerFilter(__FUNCTION__, __LINE__, 'handle_view_php', 'HANDLE_REFERRER_BANNER_VIEW');

	// Generic filter to add hidden fields to formulars
	registerFilter(__FUNCTION__, __LINE__, 'open_form_fields', 'ADD_INPUT_HIDDEN_SESSION_ID');
}

// "Registers" a new filter function
function registerFilter ($F, $L, $filterName, $filterFunction, $silentAbort = TRUE, $force = FALSE, $isDryRun = FALSE) {
	// Extend the filter function name
	$filterFunction = 'FILTER_' . strtoupper($filterFunction);

	// Debug message with FILTER_ prefix
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'ENTRY: filterName=' . $filterName . ',filterFunction=' . $filterFunction . ',force=' . intval($force) . ',isDryRun=' . intval($isDryRun) . ',F=' . basename($F) . ',L=' . $L . ' - ENTERED!');

	// Is that filter already there?
	if ((isset($GLOBALS['cache_array']['filter']['chains'][$filterName][$filterFunction])) && ($force === FALSE)) {
		// In installation phase we always want to abort
		if (($silentAbort === FALSE) || (isInstallationPhase())) {
			// Add fatal message
			reportBug(__FUNCTION__, __LINE__, sprintf("Filter chain %s has already filter function %s registered! F=%s,L=%s,force=%d", $filterName, $filterFunction, basename($F), $L, intval($force)));
		} // END - if

		// Abort here
		return FALSE;
	} // END - if

	// Shall we add it?
	if ($isDryRun === FALSE) {
		// Is the function there?
		if (!function_exists($filterFunction)) {
			// Then abort here
			logDebugMessage(__FUNCTION__, __LINE__, sprintf("Filter function %s could not be added to filter chain %s. F=%s,L=%s,force=%d", $filterFunction, $filterName, basename($F), $L, intval($force)));
			return FALSE;
		} // END - if

		// Simply add it to the array
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'REGISTER: filterName=' . $filterName . ',filterFunction=' . $filterFunction . ',force=' . intval($force) . ',F=' . basename($F) . ',L=' . $L);
		$GLOBALS['cache_array']['filter']['chains'][$filterName][$filterFunction] = 'A';
		$GLOBALS['cache_array']['filter']['counter'][$filterName][$filterFunction] = '0';
	} // END - if

	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'ENTRY: filterName=' . $filterName . ',filterFunction=' . $filterFunction . ',force=' . intval($force) . ',isDryRun=' . intval($isDryRun) . ',F=' . basename($F) . ',L=' . $L . ' - EXIT!');

	// Worked
	return TRUE;
}

// "Unregisters" a filter from the given chain
function unregisterFilter ($F, $L, $filterName, $filterFunction, $force = FALSE, $isDryRun = FALSE) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'F=' . $F . ',L=' . $L . ',filterName=' . $filterName . ',filterFunction=' . $filterFunction . ',force=' . intval($force) . ',isDryRun=' . intval($isDryRun) . ' - ENTERED!');

	// Extend the filter function name only if not loaded from database
	if (!isset($GLOBALS['cache_array']['filter']['loaded'][$filterName][$filterFunction])) {
		$filterFunction = 'FILTER_' . strtoupper($filterFunction);
	} // END - if

	// Is that filter there?
	if ((!isset($GLOBALS['cache_array']['filter']['chains'][$filterName][$filterFunction])) && ($force === FALSE)) {
		// Not found, so abort here
		logDebugMessage(__FUNCTION__, __LINE__, sprintf('Filter function %s cannot be unregistered from filter %s.', $filterFunction, $filterName));
		return FALSE;
	} // END - if

	// Shall we remove? (default, not while just showing an extension removal)
	if (($isDryRun === FALSE) && (isset($GLOBALS['cache_array']['filter']['chains'][$filterName][$filterFunction]))) {
		// Mark for filter removal
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'F=' . $F . ',L=' . $L . ',filterName=' . $filterName . ',filterFunction=' . $filterFunction . ' - REMOVE!');
		$GLOBALS['cache_array']['filter']['chains'][$filterName][$filterFunction] = 'R';
	} // END - if

	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'F=' . $F . ',L=' . $L . ',filterName=' . $filterName . ',filterFunction=' . $filterFunction . ',force=' . intval($force) . ',isDryRun=' . intval($isDryRun) . ' - EXIT!');

	// Worked
	return TRUE;
}

// "Runs" the given filters, filterData is optional and can be any type of data
function runFilterChain ($filterName, $filterData = NULL) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'filterName=' . $filterName . ',filterData[]=' . gettype($filterData) . ' - ENTERED!');

	// Is that filter chain there?
	if (!isset($GLOBALS['cache_array']['filter']['chains'][$filterName])) {
		// Log not found filters in debug-mode
		if (isDebugModeEnabled()) {
			// Log it away...
			logDebugMessage(__FUNCTION__, __LINE__, 'Filter chain ' . $filterName . ' does not exist.');
		} // END - if

		// Abort here and return content
		return $filterData;
	} // END - if

	// Default return value
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'filterName=' . $filterName . ',count()=' . count($GLOBALS['cache_array']['filter']['chains'][$filterName]));
	$returnValue = $filterData;

	// Then run all filters
	foreach ($GLOBALS['cache_array']['filter']['chains'][$filterName] as $filterFunction => $active) {
		// Debug message
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Running: name=' . $filterName . ',func=' . $filterFunction . ',active=' . $active);

		// Is the filter active or newly added??
		if (($active == 'Y') || ($active == 'A') || ((in_array($filterName, array('shutdown', 'extension_remove', 'post_extension_run_sql'))) && ($active == 'R'))) {
			// Is this filter there?
			if (!function_exists($filterFunction)) {
				// Should be fixed
				reportBug(__FUNCTION__, __LINE__, 'filterName=' . $filterName . ',filterFunction=' . $filterFunction . ' - AUTO-UNREGISTERED!');
			} // END - if

			// Call the filter chain
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, $filterName . '/' . $filterFunction . ',[]=' . gettype($returnValue) . ' - CALLING!');
			$returnValue = call_user_func_array($filterFunction, array($returnValue));

			// Update usage counter
			countFilterUsage($filterName, $filterFunction);
		} elseif (isDebugModeEnabled()) {
			// Debug message
			logDebugMessage(__FUNCTION__, __LINE__, 'Skipped: name=' . $filterName . ',func=' . $filterFunction . ',active=' . $active);
		}
	} // END - foreach

	// Return the filtered content
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'filterName=' . $filterName . ',filterData[]=' . gettype($filterData) . ',returnValue[]=' . gettype($returnValue) . ' - EXIT!');
	return $returnValue;
}

// Count the filter usage
function countFilterUsage ($filterName, $filterFunction) {
	// Is it there?
	if (isset($GLOBALS['cache_array']['filter']['counter'][$filterName][$filterFunction])) {
		// Yes, then increase
		$GLOBALS['cache_array']['filter']['counter'][$filterName][$filterFunction]++;
	} else {
		// No, then create
		$GLOBALS['cache_array']['filter']['counter'][$filterName][$filterFunction] = 1;
	}
}

// Prepares the filter array for usage
function prepareFilterArray () {
	// Abort here if array is absend (e.g. not cached)
	if (!isset($GLOBALS['cache_array']['filter']['filter_name'])) {
		// Abort silently
		return FALSE;
	} // END - if

	// Init dummy array
	$filterArray = array(
		'chains'  => array(),
		'loaded'  => array(),
		'counter' => array()
	);

	// Found in cache so rewrite the array
	foreach ($GLOBALS['cache_array']['filter']['filter_name'] as $idx => $filterName) {
		// Get filter function
		$filterFunction = $GLOBALS['cache_array']['filter']['filter_function'][$idx];

		// Add the element with mapped index
		$filterArray['counter'][$filterName][$filterFunction] = $GLOBALS['cache_array']['filter']['filter_counter'][$idx];
		$filterArray['loaded'][$filterName][$filterFunction]  = TRUE;
		$filterArray['chains'][$filterName][$filterFunction]  = $GLOBALS['cache_array']['filter']['filter_active'][$idx];
	} // END - foreach

	// Remove the cache
	$GLOBALS['cache_array']['filter'] = $filterArray;
}

/**
 * Loads filter for given extension if present. This function will silently
 * ignore absent filter files.
 *
 * @param	$ext_name	Name of extension
 * @return	void
 */
function loadExtensionFilters ($ext_name) {
	// Debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'ext_name=' . $ext_name . ' - ENTERED!');

	// Is there a cache entry?
	if (!isset($GLOBALS[__FUNCTION__][$ext_name])) {
		// Default is not found
		$GLOBALS[__FUNCTION__][$ext_name] = FALSE;

		// Construct include file name
		$incFileName = sprintf("inc/filter/%s_filter.php", $ext_name);

		// Is the include file readable?
		if (isIncludeReadable($incFileName)) {
			// Load the include file
			loadIncludeOnce($incFileName);

			// Mark the file as loaded
			$GLOBALS[__FUNCTION__][$ext_name] = TRUE;
		} elseif (isDebugModeEnabled()) {
			// Log missing file
			//* NOISY-DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Filter include file ' . $incFileName . ' for extension ' . $ext_name . ' is missing.');
		}
	} // END - if

	// Debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'ext_name=' . $ext_name . ',result=' . intval($GLOBALS[__FUNCTION__][$ext_name]) . ' - EXIT!');
}

// [EOF]
?>
