<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/01/2008 *
 * ===================                          Last change: 10/01/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : removeip_functions.php                           *
 * -------------------------------------------------------------------- *
 * Short description : Special functions for removeip extension         *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Spezielle Funktion fuer removeip-Erweiterung     *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// "Getter" for anonymous remote IP number
function getAnonymousRemoteAddress ($remoteAddr) {
	// Is config enabled?
	if (getConfig('removeip_anon_ip') == 'Y') {
		// Switch way do we like?
		switch (getConfig('anonymous_ip')) {
			case 'LOCAL': // localhost
				$remoteAddr = '127.0.0.1';
				break;

			case 'ZERO': // All four numbers zero
				$remoteAddr = '0.0.0.0';
				break;

			case 'RANDOM': // Generate random ip
				$remoteAddr = mt_rand(1, 254) . '.' . mt_rand(0, 254) . '.'.mt_rand(0, 254) . '.' . mt_rand(1, 254);
				break;
		} // END - switch
	} // END - if

	// Return it
	return $remoteAddr;
}

// "Getter" for anonymous remote hostname
function getAnonymousRemoteHost ($remoteHost) {
	// Is config enabled?
	if (getConfig('removeip_anon_host') == 'Y') {
		// Set anon hostname
		$remoteHost = 'localhost.localnet';
	} // END - if

	// Return it
	return $remoteHost;
}

// "Getter" for anonymous user agent
function getAnonymousUserAgent ($userAgent) {
	// Is config enabled?
	if (getConfig('removeip_anon_ua') == 'Y') {
		// Set anon user agent
		$userAgent = '-';
	} // END - if

	// Return it
	return $userAgent;
}

// "Getter" for anonymous referer
function getAnonymousReferer ($referer) {
	// Is config enabled?
	if (getConfig('removeip_anon_ref') == 'Y') {
		// Set anon user agent
		$referer = '-';
	} // END - if

	// Return it
	return $referer;
}

// Adds informations about anonymity/privacy to the menu
function addAnonymityLevel () {
	// "Base-privacy" is by default low (we add more later)
	$anonymity = '0';

	// Is some data anonymized?
	if (getConfig('removeip_anon_ip')   == 'Y') $anonymity++;
	if (getConfig('removeip_anon_host') == 'Y') $anonymity++;
	if (getConfig('removeip_anon_ua')   == 'Y') $anonymity++;
	if (getConfig('removeip_anon_ref')  == 'Y') $anonymity++;

	// Calculate anonymity level
	$level = round($anonymity / 4 * 3);

	// Set constant name suffix depending on that level
	$suffix = 'unsupported_' . $level;
	switch ($level) {
		case '0':
			$suffix = 'none';
			break;

		case '1':
			$suffix = 'low';
			break;

		case '2':
			$suffix = 'medium';
			break;

		case '3': // High level
			$suffix = 'high';
			break;
	} // END - while

	// Default message
	$message = sprintf("{--REMOVEIP_LEVEL_%s--}", strtoupper($suffix));

	// Output message in template
	return loadTemplate('removeip_level', TRUE, $message);
}

// [EOF]
?>
